unit Unit_USNO;


interface

uses
     Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     StdCtrls, Menus,printers;


procedure read_UCAC4(filename:string;epoch2:real;var ra,dec: real; var  mag_UCAC, star_nr: integer);{Read UCAC4 star database}
var  UCAC_class  : shortint; {start of double star}
     UCAC_leda_diameter  : shortint; {log10 object diameter in 0.1  arcmin}
     UCAC_X2M_diameter  : shortint; {X2M diameter 0.1  arcmin}
     UCAC_poor_PM: shortint; {proper motion capped}
var
   mass_id_nr:  integer;
   USNOfile_open   : integer;
   recordposition : integer;

implementation
uses D290conv;

type
  UCAC4hdr = packed record  {version 2013, packed required in delphi 7 to get correct length}

{ 1  ra     I*4 mas}        RAU  : integer;     //right ascension at  epoch J2000.0 (ICRS)  (1)
{ 2  spd    I*4 mas}        DECU : integer;     //south pole distance epoch J2000.0 (ICRS)  (1)
{ 3  im1    I*2 millimag}   U4Fmag: smallint;   //UCAC fit model magnitude                  (2)
{ 4  im2    I*2 millimag}   U4Amag: smallint;   //UCAC aperture  magnitude                  (2)
{ 5  sigmag I*1  1/100 mag} U4REag: shortint;   //UCAC error on magnitude (larger of sc.mod)(3)
{ 6  objt   I*1} 	          U4object: shortint; //object type                               (4)
{ 7  dsf    I*1}            U4doubleflag: shortint;// double star flag                          (5)
// 15 bytes

{ 8  sigra  I*1 mas}        e_RAm: shortint;    //s.e. at central epoch in RA (*cos Dec)
{ 9  sigdc  I*1 mas}        e_DEm: shortint;    //s.e. at central epoch in Dec
{10  na1    I*1}            na1  : shortint;    //total # of CCD images of this star
{11  nu1    I*1}            nrCCD: shortint;    // # of CCD images used for this star        (6)
{12  us1    I*1}            ncatepoch : shortint;// # catalogs (epochs) used for proper motions
// 5bytes

{13  cepra  I*2 0.01 yr}    EpRAm1900: smallint; //central epoch for mean RA, minus 1900
{14  cepdc  I*2 0.01 yr}    EpDEm1900: smallint; //central epoch for mean Dec,minus 1900
{15  pmrac  I*2 0.1 mas/yr} pmRA : smallint;      //proper motion in RA*cos(Dec)
{16  pmdc   I*2 0.1 mas/yr} pmDE : smallint;      //proper motion in Dec
{17  sigpmr I*1 0.1 mas/yr} e_pmRA: shortint;    //s.e. of pmRA * cos Dec
{18  sigpmd I*1 0.1 mas/yr} e_pmDE: shortint;    //s.e. of pmDec
// 10 bytes

{19  id2m   I*4}            mass_id: integer;    //2MASS unique star identifier
{20  jmag   I*2 millimag}   mag_J  : smallint;   //2MASS J  magnitude
{21  hmag   I*2 millimag}   mag_K  : smallint;   //2MASS H  magnitude
{22  kmag   I*2 millimag}   mag_H  : smallint;   //2MASS K_s magnitude
{23  icqflg I*1 * 3}        icqflg1: shortint;   //2MASS cc_flg*10 + phot.qual.flag J        (7)
{24}                        icqflg2: shortint;   //2MASS cc_flg*10 + phot.qual.flag H        (7)
{25}                        icqflg3: shortint;   //2MASS cc_flg*10 + phot.qual.flag K_s      (7)
{26  e2mpho I*1 * 3}        e2mpho1: shortint;   //2MASS error photom. J (1/100 mag)         (8)
{27}                        e2mpho2: shortint;   //2MASS error photom. H(1/100 mag)          (8)
{28}                        e2mpho3: shortint;   //2MASS error photom. K_s(1/100 mag)        (8)
// 16 bytes

{29         I*2 millimag}   APASS_B: smallint;   //B magnitude from APASS
{30         I*2 millimag}   APASS_V: smallint;   //V magnitude from APASS
{31         I*2 millimag}   APASS_g: smallint;   //g magnitude from APASS
{32         I*2 millimag}   APASS_r: smallint;   //r magnitude from APASS
{33         I*2 millimag}   APASS_i: smallint;   //i magnitude from APASS
{34   apase  I*1 1/100 mag} apas_EB: shortint;  // error of B magnitude from APASS         (14)
{35   apase  I*1 1/100 mag} apas_EV: shortint;  // error of V magnitude from APASS         (14)
{36   apase  I*1 1/100 mag} apas_Eg: shortint;  // error of g magnitude from APASS         (14)
{37   apase  I*1 1/100 mag} apas_Er: shortint;  // error of r magnitude from APASS         (14)
{38   apase  I*1 1/100 mag} apas_Ei: shortint;  // error of i magnitude from APASS         (14)
{39  g1     I*1}            yale_g1: shortint;  //  Yale SPM g-flag*10  c-flag
// 16  bytes

{40 63-66 icf(1) I*4}       icf_info: integer;   //  FK6-Hipparcos-Tycho source flag         (16)
{41       icf(2) .. }                            //  AC2000       catalog match flag         (17)
{42       icf(3) .. }                            //  AGK2 Bonn    catalog match flag         (17)
{43       icf(4) .. }                            //  AKG2 Hamburg catalog match flag         (17)
{44       icf(5) .. }                            //  Zone Astrog. catalog match flag         (17)
{45       icf(6) .. }                            //  Black Birch  catalog match flag         (17)
{46       icf(7) .. }                            //  Lick Astrog. catalog match flag         (17)
{47       icf(8) .. }                            //  NPM  Lick    catalog match flag         (17)
{48       icf(9) .. }                            //  SPM  YSJ1    catalog match flag         (17)
//  4 bytes

{49 67    leda   I*1}       leda  : shortint;    //  LEDA galaxy match flag                  (18)
{50 68    x2m    I*1}       x2m   : shortint;    //  2MASS extend.source flag                (19)
{51 69-72 rnm    I*4}       rnm   : integer;     //  unique star identification number       (20)
{52 73-74 zn2    I*2}       zn2   : smallint;    //  zone number of UCAC2 (0 = no match)     (21)
{53 75-78 rn2    I*4}       rn2   : integer;     //  running record number along UCAC2 zone  (21)
// 12 bytes

{    78 = total number of bytes per star record}
end;{UCAC4 record}



var
   bufc4: array[1..sizeof(UCAC4hdr)] of byte;
   puc4      : ^UCAC4hdr;		        { pointer to UCAC record }

var
   TheFile_USNO   : TfileStream;
   Reader_USNO    : TReader;
   nr_records     : integer;
   filenameUSNO   : string; //array[0..120] of char;

const
   nrread        :integer=0;


procedure read_UCAC4(filename:string;epoch2:real;var ra,dec: real; var  mag_UCAC, star_nr: integer);{Read UCAC4 star database}
var
  tempstr1, tempstr2 :string[7];
begin
   {$I-}
   repeat
    if  ( (USNOfile_open=0) )  then    {file_open omdat anders fileroutine soms vastloopt}
    begin
      try
      TheFile_USNO :=tfilestream.Create( filename, fmOpenRead );
      except
        beep;
        USNOfile_open:=0;
        mag_UCAC:=10000;{stop further reading}
        exit;
      end;

      nr_records:= trunc(thefile_USNO.size/sizeof(UCAC4hdr));
      Reader_USNO := TReader.Create (TheFile_USNO , 12*1024);{number of UCAC records}
      {thefile3.size-reader.position>sizeof(hnskyhdr) could also be used but slow down a factor of 2 !!!}
      USNOfile_open:=2; {buffer size is .. x 1024}
    end;
    reader_USNO.read(bufc4,sizeof(UCAC4hdr));
    {Note: pointer pu2 is set in program part}
    with puc4^ do
       begin {correct star movement for new epoch. Coordinates are always in equinox 2000}

         DEC {(epoch T)} := (DECU -90/360000 + pmDE * (epoch2 - 2000.0)/10.0)*((pi/180)/3600000)-pi/2;  {DEC first}
         RA  {(epoch T)} := (RAU  + pmRA / cos(dec)*(epoch2 - 2000.0)/10.0)*((pi/180)/3600000);  {cos new !!!!}

          {note if pmDe and pmRA is 32767 the real PM of that star is larger and found in the extra table  file  u4hpm.dat (32 stars, ASCII).}


         {2015 practical magnitude selection, take brightest}
  {1}   mag_Ucac:=u4Fmag;{accurate for faint stars}
  {2}   if APASS_V < mag_Ucac then mag_Ucac:=APASS_V; {mag=2000 if no magn given, use alternative, for bright stars APASS_V is much better}
  {3}   if U4Amag  < mag_Ucac then mag_Ucac:=U4Amag;  {some faint stars have no u4Fmag}
         mag_Ucac:=round(mag_Ucac/100); {size mili magn to 1/10 magnitude}

        star_nr:=recordposition+1;{star number in zone, at record position 0, star nr 1 is located}
                                                             // 4UCzzz-nnnnnn
                                                            // The "4UC" is constant and indicates the UCAC4 catalog. The 3 digit "zzz"
                                                            // number is the zone the star is in, followed by a dash and a 6 digit
                                                            // number which is the record number of the star on that zone.

          mass_id_nr:=mass_id;{2mass id nr}
     // UCAC4	367-016700 , Sirus
     // if star_nr= 446136122 then
     //                                writeln;
     {simbad query:}
     {polaris, ICRS coord. (ep=J2010, eqJ2000) : 	02 31 49.09456 +89 15 50.7923 }
     {polaris, ICRS coord. (ep=J2016, eqJ2000) : 	02 31 52.78845 +89 15 50.6026 }
     {polaris, ICRS coord. (ep=J2100, eqJ2000) : 	02 32 12.17295 +89 15 49.6035 }
     {polaris, ICRS coord. (ep=J2250, eqJ2000) : 	02 32 46.75267 +89 15 47.8064 }

     {sirius, ICRS coord. (ep=J2000, eqJ2000) 06 45 08.91728 -16 42 58.0171 }
     {sirius, ICRS coord. (ep=J2016, eqJ2000) 06 45 08.30913 -16 43 17.5868 }
     {sirius, ICRS coord. (ep=J2100, eqJ2000) 06 45 05.11511 -16 45 00.3480 }
     {sirius, ICRS coord. (ep=J2250, eqJ2000) 06 44 59.40628 -16 48 03.9339 }

     {capella, ICRS coord. (ep=J2100, eqJ2000) 05 16 41.35871 +45 59 52.7693 }
     {capella, ICRS coord. (ep=J2016, eqJ2000) 05 16 41.47425 +45 59 45.9390 }
     {capella, ICRS coord. (ep=J2100, eqJ2000) 05 16 42.08070 +45 59 10.0801 }
     {capella, ICRS coord. (ep=J2250, eqJ2000) 05 16 43.16312 +45 58 06.0459 }

          UCAC_class:=U4doubleflag;
          UCAC_leda_diameter:=leda;{log10 object diameter in 0.1  arcmin}
          UCAC_X2M_diameter :=X2M; {X2M diameter 0.1  arcmin}


          if ((pmRA=32767) or (pmDE=32767)) then UCAC_poor_PM:=1 else UCAC_poor_PM:=0;         //Note  (8):  A value of 32767 for either proper motion component means
                                                                                              //the real PM of that star is larger and found in the extra table file  u4hpm.dat (32 stars, ASCII
                                                                                              // example UCAC4 225005836, Kapteyns star

         {special for TYCHO with UCAC4 supplement combination}
         if icf_info>=100000000 then {FK6-Hipparcos-Tycho source flag}
           if main_form.box_tycho2.checked=true then mag_UCAC:=99999; {skip this star, already written by TYCHO routine}

     end;
     inc(recordposition);


     if recordposition>=nr_records then
     begin
        Reader_USNO.free;
        TheFile_USNO.free;
        recordposition:=0;
        USNOfile_open:=0;
        exit;
     end;
   until ((mag_UCAC>round(mag_begin*10))  and  (mag_UCAC<=round((mag_begin+step)*10)));{magnitude filter, round function to get sharp selection and not star is forgotten or double processed}
end;


begin  {program}
  USNOfile_open:=0;
  puc4:= @bufc4[1];	{ set UCAC pointer for read routine }
end.
